<?php

namespace Quivers\Integration\Block\Sales\Adminhtml\Order;
class IncludedTax extends \Magento\Framework\View\Element\Template
{
    /**
     * Tax configuration model
     *
     * @var \Magento\Tax\Model\Config
     */
    protected $_config;

    /**
     * @var Order
     */
    protected $_order;

    /**
     * @var \Magento\Framework\DataObject
     */
    protected $_source;
    protected $_scopeConfig;
    /**
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\Tax\Model\Config $taxConfig
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Tax\Model\Config $taxConfig,
        array $data = []
    ) {
        $this->_config = $taxConfig;
        $this->_scopeConfig = $scopeConfig;
        $this->_checkoutSession = $checkoutSession;
        parent::__construct($context, $data);
    }

    /**
     * Check if we nedd display full tax total info
     *
     * @return bool
     */
    public function displayFullSummary()
    {
        return true;
    }

    /**
     * Get data (totals) source model
     *
     * @return \Magento\Framework\DataObject
     */
    public function getSource()
    {
        return $this->_source;
    } 
    public function getStore()
    {
        return $this->_order->getStore();
    }

      /**
     * @return Order
     */
    public function getOrder()
    {
        return $this->_order;
    }

    /**
     * @return array
     */
    public function getLabelProperties()
    {
        return $this->getParentBlock()->getLabelProperties();
    }

    /**
     * @return array
     */
    public function getValueProperties()
    {
        return $this->getParentBlock()->getValueProperties();
    }

    /**
     * Initialize all order totals relates with tax
     *
     * @return \Magento\Tax\Block\Sales\Order\Tax
     */
     public function initTotals()
    {
        $parent = $this->getParentBlock();
        if($parent){
            $this->_order = $parent->getOrder();
            $this->_source = $parent->getSource();
            $includedTax = $this->_source->getQuiversIncludedTax();
            if ($includedTax > 0){
                $included_tax = new \Magento\Framework\DataObject(
                    [
                        'code' => 'included_tax',
                        'strong' => false,
                        'value' =>  $this->_source->getQuiversIncludedTax(),
                        'label' => __('Estimated Included Tax'),
                    ]
                );

                $parent->addTotal($included_tax, 'included_tax');
            }
        }
        return $this;
    }
    /**
     * @return array
     */
    public function getsitekeys()
    {
      return  $this->_checkoutSession->getSiteKey()['site_key_secret'];
    }

    /**
     * @return array
     */
    public function getSiteKeysAPI(){
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $quiversSandbox = $this->_scopeConfig->getValue("quivers/parameters/quivers_sanndbox", $storeScope);
    
        if($quiversSandbox == 0) {
            $middlewareurl ='https://middleware.quivers.com/profile/';
        }
        else {
            $middlewareurl ='https://middleware.quiversdemo.com/profile/';
        }
 
        $ref_id = $this->_scopeConfig->getValue("quivers/parameters/merchant_ref_id", $storeScope);
        $this->curlClient->setOption(CURLOPT_CONNECTTIMEOUT, 9);
        $this->curlClient->setOption(CURLOPT_TIMEOUT, 10);
        $sitedata = [
            "client_type"=> "Magento2",
            "business_refid"=> $ref_id
        ];
        try {
            $this->curlClient->post($middlewareurl.'retrieve/sitekeys/',json_encode( $sitedata));
            $response = $this->curlClient->getBody();
            $this->_checkoutSession->setSiteKey(json_decode($response,true));
        
        } catch (\Exception $e) {
            throw new \Magento\Framework\Validator\Exception(__($e->getMessage().'Site key APIs not working please check again.'));
        }
    }

}

