<?php 

namespace Quivers\Integration\Model;
use Magento\Payment\Model\CcGenericConfigProvider;
use Magento\Payment\Model\CcConfig;
use Stripe\Stripe;

/**
 * ConfigProvider Class
 */
class ConfigProvider extends CcGenericConfigProvider
{

	const CODE = 'quivers_integration';
	const PATH = 'ecommerce';

    protected $scopeConfig;

	public function __construct(
	    CcConfig $ccConfig,
	    \Magento\Payment\Helper\Data $paymentHelper,
	    \Magento\Checkout\Model\Session $checkoutSession,
	    \Magento\Customer\Model\Session $customerSession,
	    \Magento\Payment\Model\Config $paymentConfig,
	    \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
	    \Magento\Framework\Message\ManagerInterface $messageManager,
	    array $methodCodes = []
	) {
	    // $this->paymentHelper    = $paymentHelper;
	    // $this->checkoutSession  = $checkoutSession;
	    $this->customerSession  = $customerSession;
	    // $this->paymentConfig    = $paymentConfig;
	    $this->scopeConfig = $scopeConfig;
	    $this->messageManager = $messageManager;
		$storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
	    parent::__construct($ccConfig, $paymentHelper, [static::CODE]);
	}

	/**
	 * Get checkout config.
	 *
	 * @return array
	 */
	public function getConfig()
	{	
		$storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $publicKey = $this->scopeConfig->getValue('payment/quivers_integration/public_key', $storeScope);
        $title = $this->scopeConfig->getValue('payment/quivers_integration/title', $storeScope);
        $payPalTitle = $this->scopeConfig->getValue('payment/quivers_integration_paypal/title', $storeScope);
        $payPalClientId = $this->scopeConfig->getValue('payment/quivers_integration_paypal/client_id', $storeScope);
        $payPalSandbox = $this->scopeConfig->getValue('payment/quivers_integration_paypal/quivers_paypal_sanndbox', $storeScope);
        $active = $this->scopeConfig->getValue('quivers/parameters/active', $storeScope);
		$quiversSandbox = $this->scopeConfig->getValue('quivers/parameters/quivers_sanndbox', $storeScope);
		$customer = $this->customerSession->getCustomer();
		$customerId = $customer->getId();
		$stripeCustomerId = $customer->getQuiversStripeCustomerId();
		try{
			$customersList = \Stripe\Customer::all();
			
			foreach($customersList as $customerKey){
				$livemode = $customerKey->livemode;
				if($quiversSandbox == 1 && $livemode == true)
				{
					unset($customersList);
				}
			}
		}catch (\Exception $e) {
			$customersList = array();
		}
		try{
		if(empty($customersList)){
			$stripeCustomerId = '';
		}
		$stripeCards = [];
		$selectedCard = [];
		$paymentmethods =[];
		if($stripeCustomerId) {
			$allpaymentmethod=\Stripe\PaymentMethod::all(
					[
						'customer' => $stripeCustomerId,
				  		'type' => 'card'
				  	]);
			foreach($allpaymentmethod->data as $paymentmethod) 
				{
					if(explode("_", $paymentmethod->id)[0]=="pm")
						 	{	$paymentmethod_detail=[
						 			'payment_method_id'=>$paymentmethod->id,
						 			'exp_month'=>$paymentmethod->card->exp_month,
						 			'exp_year'=>$paymentmethod->card->exp_year,
						 			'last4'=>$paymentmethod->card->last4,
						 			'funding'=>$paymentmethod->card->funding,
	        						'country'=>$paymentmethod->card->country
						 				];
						 		$paymentmethods[]=$paymentmethod_detail;
						 	}
				}
			$stripeCustomer = \Stripe\Customer::retrieve($stripeCustomerId);
	        if($stripeCustomer) {
	        	$savedCardIds = array_map("trim", explode(',', $customer->getQuiversStripeCardIds()));
	        	if($stripeCustomer->id===$stripeCustomerId && $stripeCustomer->sources) {
	        		$cards = $stripeCustomer->sources->data;
	        		if(count($cards)>0) {
	        			foreach($cards as $card) {
	        				$passed=false;
	        				if(isset($card->card) && in_array($card->id, $savedCardIds)) {
	        					foreach($paymentmethods as $pm)
	        					{	
	        						if(
	        							$pm["exp_month"]==$card->card->exp_month && 
	        							$pm["exp_year"]==$card->card->exp_year && 
	        							$pm["last4"]==$card->card->last4 &&
	        							$pm["funding"]==$card->card->funding &&
	        							$pm["country"]==$card->card->country
	        						)
	        						{	$passed=true;
	        							break;
	        						}
	        					}
	        					if(!$passed)
	        					{	
	        						continue;
	        					}
	        					$details = [
		        					'paymentSourceId'=>$card->id,
		        					'last4' => $card->card->last4,
		        					'expMonth' => $card->card->exp_month,
		        					'expYear' => $card->card->exp_year,
		        					'country' => $card->card->country,
		        					'cardType' => $card->card->funding,
		        					'label' => "XXXX-XXXX-XXXX-".$card->card->last4.' '.$card->card->exp_month.'/'.$card->card->exp_year,
		        				];
		        				$stripeCards[] = $details;
		        				if($card->id===$stripeCustomer->default_source) {
		        					$selectedCard = $details;
		        				}
	        				}
	        			}
	        		}
	        	}
	        }
		}
		$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
		$storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
		$baseUrl=$storeManager->getStore()->getBaseUrl();
		if(!isset($baseUrl)){
			 throw new \Magento\Framework\Validator\Exception(__());
		}
		$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $customer_session=$objectManager->get('\Magento\Customer\Model\Session');
		$config = parent::getConfig();
		$config = array_merge_recursive($config, [
			'validation' => $active ,
            'payment' => [
                static::CODE => [
					'publicKey' => $publicKey,
					'siteKey' =>isset($customer_session->getSiteKey()['site_key_secret'])?$customer_session->getSiteKey()['site_key_secret']:0,
                    'title'=> $title,
                    'payPalTitle' => $payPalTitle,
                    'payPalClientId' => $payPalClientId,
                    'payPalSandbox' => $payPalSandbox,
                    'stripeCards' => $stripeCards,
                    'selectedCard' => $selectedCard,
                    'customerId' => $customerId,
	                'baseUrl' => $baseUrl,
	                'paymentmethods'=> $paymentmethods,
	                'path'=>static::PATH
                ],
            ],
        ]);
        return $config;
	}
	catch (\Exception $e) {
		$this->messageManager->addErrorMessage(__("Sorry! Looks like the Payment Method is not yet available to use because it has not yet been Fully Configured."));
        throw new \Magento\Framework\Validator\Exception(__('Stripe public key and secret keys are Invalid , Get these from your Stripe account API settings.'));}
}

}