<?php

namespace Quivers\Integration\Model\Total\Quote;

use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\CouldNotSaveException;
use function PHPSTORM_META\type;
use Quivers\Integration\LogTracking;
use Stripe\Stripe;

class QuiversTax extends \Magento\Tax\Model\Sales\Total\Quote\Tax {
    //print_r("**********************");
    
    const XML_MARKETPLACE_ID = 'quivers/parameters/marketplaceId';
    const XML_QUIVERS_SANDBOX = 'quivers/parameters/quivers_sanndbox';
    const XML_USD_MARKETPLACE_ID = 'quivers_tax/marketplaces/usdMarketPlaceId';
    const XML_EUR_MARKETPLACE_ID = 'quivers_tax/marketplaces/eurMarketPlaceId';
    const XML_CAD_MARKETPLACE_ID = 'quivers_tax/marketplaces/cadMarketPlaceId';
    const XML_AUD_MARKETPLACE_ID ='quivers_tax/marketplaces/audMarketPlaceId';
    const XML_JPY_MARKETPLACE_ID = 'quivers_tax/marketplaces/jpyMarketPlaceId';
    const XML_GBP_MARKETPLACE_ID = 'quivers_tax/marketplaces/gbpMarketPlaceId';    
    const XML_ENABLED = 'quivers/parameters/active';

    private $logger;
    protected $curlClient;
    protected $scopeConfig;

    public function collect(
        \Magento\Quote\Model\Quote $quote,
        \Magento\Quote\Api\Data\ShippingAssignmentInterface $shippingAssignment,
        \Magento\Quote\Model\Quote\Address\Total $total)
    {   

        $items = $shippingAssignment->getItems();
        if (!count($items)) {
             return $this;
        }
        parent::collect($quote, $shippingAssignment, $total);

        $startTime = microtime(true);
        $taxEndpoint = '/CustomerOrders/Validate';
        $countriesEndpoint = '/countries';
        $logger = \Magento\Framework\App\ObjectManager::getInstance()->get(\Psr\Log\LoggerInterface::class);
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $curlClient = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\HTTP\Client\Curl::class);
        $log = new LogTracking($scopeConfig);
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        // define a functin to get markeplace id according to order currency
        $quiversSandbox = $scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $storeScope);
        $active = $scopeConfig->getValue(self::XML_ENABLED, $storeScope);
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $customer_session=$objectManager->get('\Magento\Customer\Model\Session');
        // Quivers Setting Form configuration decides the environment(either test/demo or prod).
        if ($active == 1){
        if($quiversSandbox == 0) {
            $baseUrl = 'https://api.quivers.com/v1';
        }
        else {
            $baseUrl = 'https://api.quiversdemo.com/v1';
        }
        $taxUrl = $baseUrl.$taxEndpoint;

        $items = array();
        $productMapping = [];


        # if no item selected then no need to proceed further.
        if(count($quote->getAllItems()) <= 0) {
            return $this;
        }

        // No need to calculate tax if user has not specified the shipping address.
        $shipping_address = $quote->getShippingAddress();
        if(!$shipping_address) {
            return $this;
        }
        $country = $shipping_address->getCountryId();
        // check the required address values.
        if(!$country) {
            return $this;
        }

        $region = $shipping_address->getRegionCode();
        if(!$shipping_address->getRegionCode()) {
            $region = 'N/A';
        }

        // Get configured marketplace of order currency.
        $marketPlace = $this->getMarketPlaceForCurrency($quote->getQuoteCurrencyCode());
        if(!$marketPlace) {
            return $this;
        }

        $shippingFeesTotal =  0.0;
        $shippingFeePerItem = 0.0;
        $shipping_amt_diff = 0.0;
        $shippingMethod = $shipping_address->getShippingMethod();
        // base currency to order currency conversion rate.
        $convRate = $quote->getBaseToQuoteRate();
        $shippingRateData = $shipping_address->getShippingRatesCollection()->getData();
        if(count($shippingRateData)>0) {
            foreach ($shippingRateData as $rate) { 
                if($shippingMethod == $rate['code']) {
                    $convPrice = $rate['price'] * $convRate;
                    $shippingFeesTotal = $convPrice;
                }
            }
        }

        //Get shipping amount from total if shipping rate is empty
        $totalShippingRate = $total->getShippingAmount();

        if ($shippingMethod != "" && $shippingFeesTotal <= 0){
            $convPrice = $totalShippingRate * $convRate;
            $shippingFeesTotal = $convPrice;
        }
        if ($shippingMethod != "" && $shippingFeesTotal <= 0){
            $shippingAmount = $quote->getShippingAddress()->getShippingAmount();
            $convPrice = $shippingAmount * $convRate;
            $shippingFeesTotal = $convPrice;
        }

        $totalItems = 0;
        foreach ($quote->getAllItems() as $quoteItem) {
            $itemData = $quoteItem->getData();
            if(array_key_exists('parent_item_id', $itemData) 
                && !is_null($itemData['parent_item_id'])) {
                continue;
            }
            $totalItems += 1;
        }

        //check if there is any bopis item
        $bopis_flag = false;
        foreach ($quote->getAllItems() as $quoteItem) {
            $itemData = $quoteItem->getData();
            if(array_key_exists('parent_item_id', $itemData) 
               && !is_null($itemData['parent_item_id'])) {
                continue;
            }
            $bopisdata= "";
            if(array_key_exists('bopisdata', $itemData)){
                $bopisdata = $itemData['bopisdata'];
                if($bopisdata != ""){
                    $bopisdata = json_decode($bopisdata);
                    if((isset($bopisdata->curbsidepick) && $bopisdata->curbsidepick==true) ||
                    (isset($bopisdata->instorepickup) && $bopisdata->instorepickup==true)){
                        $bopis_flag = true;
                        break;
                    }
                }
            }
        }

        $shippingFeePerItem = (float)$shippingFeesTotal/$totalItems;
        $shippingFeePerItem = floatval(substr($shippingFeePerItem, 0, strpos($shippingFeePerItem, '.') + 3));
        $shipping_amt_diff = round($shippingFeesTotal - ($shippingFeePerItem * $totalItems), 2);
        $shippingcodeperitem = '';
        foreach ($quote->getAllItems() as $quoteItem) {

            $itemData = $quoteItem->getData();
            // check for parent item, its existence means that the item represent the 
            // additional item that magento creates for every configurable product. The 
            // original placed item doesn't have any parent item.
            if(array_key_exists('parent_item_id', $itemData) 
                && !is_null($itemData['parent_item_id'])) {
                // $logger->info("parent_item exists ", ["" => $itemData]);
                continue;
            }
            $bopisShippingFeePerItem = 0;
            $shippingcodeperitem = '';
            if ($bopis_flag == true){
                $bopisShippingFeePerItem = (float)$itemData['bopis_shippingfee'];
                $shippingcodeperitem= $itemData['bopis_shippingcode'];
            }
            $itemProduct = $quoteItem->getProduct();
            $itemId = (int)$itemProduct->getId();

            $productMapping[$itemId] = $quoteItem;

            $itemName = $itemProduct->getName();
            $itemVariantName = $itemProduct->getSku();
            $options = array();
            $options = $quoteItem->getProduct()->getTypeInstance(true)->getOrderOptions($quoteItem->getProduct());
            $options_arr = [];
            if (isset($options['options']) && !empty($options['options'])) {
                foreach ($options['options'] as $option) {
                    $optionTitle = $option['label'];
                    $optionId = $option['option_id'];
                    $optionValue = $option['value'];
                array_push($options_arr, array(
                    "name" => $optionTitle,
                    "value" => array("name" => $optionValue)
                ));

                }
            }
            $unitPrice = $itemProduct->getPrice();
            if(array_key_exists('converted_price', $itemData)) {
                // if order is not placed in base currency of the store then 
                // converted_price is the price converted in order currency.
                $unitPrice = $itemData['converted_price'];
            }

            $totalDiscount = $quoteItem->getTotalDiscountAmount();

            $discounts = [];
            if($totalDiscount > 0 ) {
                array_push($discounts, [
                    "name"=> "Total Discount",
                    "amount"=> -$totalDiscount,
                ]);
            }

            $shippingFees = [];
            if($shippingFeePerItem > 0 && $bopis_flag == false) {
                $shippingFees = [
                    "name" => $shippingMethod,
                    "amount" => $shippingFeePerItem
                ];
            }
            elseif($bopisShippingFeePerItem > 0 && $bopis_flag == true){
                $shippingFees = [
                    "name" => $shippingcodeperitem,
                    "amount" => $bopisShippingFeePerItem
                ];
            }
            if(count($shippingFees) > 0 ) {
                $shippingFees = [$shippingFees];
            }

            #fetch taxCode through product Id
            $productId = (int)$quoteItem->getProductId();
            $objectManager =   \Magento\Framework\App\ObjectManager::getInstance();
            $productData = $objectManager->get('\Magento\Catalog\Api\ProductRepositoryInterface')->getById($productId);
            $productRepository = $objectManager->get('\Magento\Catalog\Model\ProductRepository');
            $_product_Object = $productRepository->get($productData->getSku());
            $taxcode = $_product_Object->getResource()->getAttribute('product_tax_category')->getFrontend()->getValue($_product_Object);
            $product = array(
                "name" => $itemName,
                "refId" => $itemId, // identifier of order item.
                "variant" => array(
                    "name" => $itemVariantName,
                    "options" => $options_arr
                )
            );
            if($taxcode){
                $product['taxCode']=$taxcode;
            }
            array_push($items, array(
            "product" => $product,
            "quantity" => $quoteItem->getQty(),
            "pricing" => array(
                    "unitPrice" => (float)$unitPrice,
                    "shippingFees" => $shippingFees,
                    "discounts"=> $discounts
                )
            ));

        }
        $taxData = [
            "marketplaceId" => $marketPlace,
            "shippingAddress" => [
                "phoneNumber" => $shipping_address->getTelephone(),
                "line1" => $shipping_address->getStreetLine(1),
                "line2" => $shipping_address->getStreetLine(2),
                "city" => $shipping_address->getCity(),
                "postCode" => $shipping_address->getPostcode(),
                "country" => $country,
                "region" => $region
            ],
            "items" => $items
        ];
        $log->session_start($taxData, date('r', $startTime)."UTC");
        // $logger->info("taxData ", ["" => $ftaxData]);
        if ($bopis_flag == false){
            foreach ($taxData["items"] as &$payload_item) {
                if (!$shipping_amt_diff) {
                    break;
                }
                $payload_item["pricing"]["shippingFees"][0]["amount"] += 0.01;
                $shipping_amt_diff -= 0.01;
            }
            unset($payload_item);
        }
        
        $curlClient->addHeader("Content-Type", "application/json");
        $curlClient->addHeader("Expect","");
        $curlClient->setOption(CURLOPT_CONNECTTIMEOUT, 9);
        $curlClient->setOption(CURLOPT_TIMEOUT, 10);
        try {
            $curlClient->post($taxUrl, json_encode($taxData));
            $request =  ["tax_url" => $taxUrl, "tax_data" => $taxData];

        } catch (\Exception $e) {
            $this->tax_calculation_via_countries_api($baseUrl, $countriesEndpoint, $quote, $total);
            $request = ["base_url" => $baseUrl, "Endpoint"=>$countriesEndpoint];
            $endTime = microtime(true);
            $log->countries_api_call($request, json_decode($curlClient->getBody()), date('r', $startTime)."UTC", date('r', $endTime)."UTC");
            $order_detail = ["Tax Data"=>$taxData, "Response"=>json_decode($curlClient->getBody())];
            $log->session_end($order_detail, date('r', $endTime)."UTC");
            return $this;
        }
        $response = $curlClient->getBody();

        $totalTax = 0;
        $payment_descriptor = "";
        $includedTax = 0;
        if($curlClient->getStatus()==200) {
            $responseDecoded = json_decode($response, true);
            // $logger->info("Response",[""=>$responseDecoded]);
            $subtotal = $responseDecoded["result"]["totals"]["subtotal"];
            $totalAmount = $responseDecoded["result"]["totals"]["total"];
            $customer_session->setTotalAmount($totalAmount);
            $payment_descriptor = $responseDecoded["result"]["statementDescriptor"];
            foreach ($responseDecoded["result"]["items"] as $responseItems) {
                $productId = $responseItems['productRefId']; // the identifier sent in request.

                $quoteItem = $productMapping[$productId]; // get the original item and update its taxes.
                $quoteItemData = $quoteItem->getData();
                $appliedTaxes = $responseItems['pricing']['taxes'];
                if(count($responseItems['pricing']['taxes']) <= 0) {
                    continue;
                }
                $tax_amount = 0;
                $tax_percent = 0;
                foreach ($appliedTaxes as $appliedTax) {
                    if($appliedTax["includedInSubtotal"]){
                        $includedTax += $appliedTax['amount'];
                        $tax_percent += 0;                        
                    }else{
                        $tax_amount += $appliedTax['amount'];
                        $tax_percent += $appliedTax['rate']*100;
                    }
                }
                if($tax_amount){
                    $totalTax += $tax_amount;
                }
                $quoteItemData['tax_amount'] = $tax_amount;
                $quoteItemData['base_tax_amount'] = $tax_amount/$convRate;
                $quoteItemData['tax_percent'] = $tax_percent;
                $quoteItem->setData($quoteItemData);
            }
            $total->setTotalAmount('tax', $totalTax);
            $total->setBaseTotalAmount('tax', $totalTax);
            // $total->setBaseGrandTotal($total->getBaseGrandTotal() + $totalTax);
            // $logger->info("Total object",[""=>$total]);
            
            //add included tax to quote
            $quote->setQuiversIncludedTax($includedTax);

            $payment = $quote->getPayment();
            $paymentKeyFromResponse = $responseDecoded["result"]["payment"];
            if(!empty($paymentKeyFromResponse) && !empty($paymentKeyFromResponse["destination"]))
            {
                $merchantId = !empty($paymentKeyFromResponse["accountId"]) ? $paymentKeyFromResponse["accountId"] : 0;
                $destination = !empty($paymentKeyFromResponse["destination"]) ? $paymentKeyFromResponse["destination"] : "" ;
                $payment->setAdditionalInformation('destination', $destination);
                $payment->setAdditionalInformation('merchantId', $merchantId);
            }
            // get payment and set payment_descriptor as additional information
            $payment->setAdditionalInformation('payment_descriptor', $payment_descriptor);
            $endTime = microtime(true);
            $log->validate_api_call($request, json_decode($response), date('r', $startTime)."UTC", date('r', $endTime)."UTC");
            $order_detail = ["Tax Data"=>$taxData, "Response"=>json_decode($response, true)];
            $log->session_end($order_detail,date('r', $endTime)."UTC");
        }
        else {
            // fallback method: if validate API fails for any reason then we calculates
            // the worst case tax for this shipping address.
            $this->tax_calculation_via_countries_api($baseUrl, $countriesEndpoint, $quote, $total);
            $request = ["base_url" => $baseUrl, "Endpoint"=>$countriesEndpoint];
            $endTime = microtime(true);
            $log->countries_api_call($request, json_decode($curlClient->getBody()), date('r', $startTime)."UTC", date('r', $endTime)."UTC");
            $order_detail = ["Tax Data"=>$taxData, "Response"=>json_decode($curlClient->getBody())];
            $log->session_end($order_detail, date('r', $endTime)."UTC");
        }
    }
        return $this;
    }

    private function tax_calculation_via_countries_api($baseUrl, $countriesEndpoint, $quote, $total) {
        $countriesUrl = $baseUrl.$countriesEndpoint;
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $curlClient = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\HTTP\Client\Curl::class);
        $curlClient->addHeader("Content-Type", "application/json");
        $curlClient->setOption(CURLOPT_CONNECTTIMEOUT, 4);
        $curlClient->setOption(CURLOPT_TIMEOUT, 5);
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $customer_session=$objectManager->get('\Magento\Customer\Model\Session');
        try {
        $curlClient->get($countriesUrl);
        } catch (\Exception $e) {
        }
        $result = $curlClient->getBody();
        if($curlClient->getStatus() != 200) {
            return;
        }
        $response = json_decode($result, true);
        $address = $quote->getShippingAddress();
        $shipping_amount = $quote->getShippingAddress()->getShippingAmount();
        $convRate = $quote->getBaseToQuoteRate();

        $tax_rate = NULL;
        if($response and isset($response['result'])){
        foreach ($response['result'] as $country) {
            if ($country['abbreviations']['two']==$address->getCountryId()) {
                foreach ($country["regions"] as $region) {
                    if ($region['abbreviation']==$address->getRegionCode()){
                        $tax_rate = $region['maxTaxRate'];
                        break;
                    }
                }
                // Quivers supports "N/A" regions for some countries.
                if (!$tax_rate && array_key_exists('0', $country['regions'])) {
                  $tax_rate = (float) $country['regions']["0"]["maxTaxRate"];
                }
            }
            if ($tax_rate) {
            break;
          }
        }
        }
        if ($tax_rate === NULL) {
            return;
        }

        $totalTax = 0; // total tax
        $subtotal = 0; // order total excluding tax
        $totalAmount = 0; // order total including tax
        $bopisShippingFee = 0.0;
        //check if there is any bopis item
        $bopis_flag = false;
        foreach ($quote->getAllItems() as $quoteItem) {
            $itemData = $quoteItem->getData();
            if(array_key_exists('parent_item_id', $itemData) 
                && !is_null($itemData['parent_item_id'])) {
                continue;
            }
            $bopisdata= "";
            if(array_key_exists('bopisdata', $itemData)){
                $bopisdata = $itemData['bopisdata'];
                if($bopisdata != ""){
                    $bopisdata = json_decode($bopisdata);
                    if((isset($bopisdata->curbsidepick) && $bopisdata->curbsidepick==true) ||
                    (isset($bopisdata->instorepickup) && $bopisdata->instorepickup==true)){
                        $bopis_flag = true;
                        break;
                    }
                }
            }
        }
        foreach ($quote->getAllItems() as $quoteItem) {
            $parent = $quoteItem->getParentItem();
            if($parent) {
                continue;
            }

            $itemDiscount = $quoteItem->getDiscountAmount();
            $itemQuantity = $quoteItem->getQty();
            $itemProduct = $quoteItem->getProduct();
            $itemUnitPrice = (float)$itemProduct->getPrice(); // item_price
            $quoteItemData = $quoteItem->getData();
            if(array_key_exists('converted_price', $quoteItemData)) {
                $itemUnitPrice = $quoteItemData['converted_price'];
            }

            //Bopis Shipping Fee
            $bopisShippingFeePerItem = 0;
            if ($bopis_flag == true){
                $bopisShippingFeePerItem = (float)$quoteItemData['bopis_shippingfee'];
            }
            $bopisShippingFee = $bopisShippingFee +$bopisShippingFeePerItem;
            $itemSubotal = $itemUnitPrice * $itemQuantity;
            $taxableAmount = $itemSubotal - $itemDiscount;
            $tax_amount = $tax_rate * $taxableAmount;
            $itemTotal = $taxableAmount + $tax_amount; // item_price_with_tax - shipping_fee_per_item

            $quoteItemData['price_incl_tax'] = $itemTotal/$itemQuantity;
            $quoteItemData['tax_amount'] = $tax_amount;
            $quoteItemData['base_tax_amount'] = $tax_amount/$convRate;
            $quoteItemData['tax_percent'] = $tax_rate * 100;
            $quoteItem->setData($quoteItemData);

            $totalTax = $totalTax + $tax_amount;
            $subtotal = $subtotal + $itemSubotal;
            $totalAmount = $totalAmount + $itemTotal;

        }

        if ($bopis_flag == false){
            $totalAmount = $totalAmount + $shipping_amount;
        }else{
            $totalAmount = $totalAmount + $bopisShippingFee;
        }

        $customer_session->setTotalAmount($totalAmount);
        $total->setTotalAmount('tax', $totalTax);
        $total->setBaseTotalAmount('tax', $totalTax);
        //$total->setBaseGrandTotal($total->getBaseGrandTotal() + $totalTax);
    }
    
    /**
     * @param \Magento\Quote\Model\Quote $quote
     * @param \Magento\Quote\Model\Quote\Address\Total $total
     * @return array
     */
    public function fetch(\Magento\Quote\Model\Quote $quote, \Magento\Quote\Model\Quote\Address\Total $total)
    { 
        $this->getSiteKeys();
        $result = parent::fetch($quote, $total);
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $active = $scopeConfig->getValue(self::XML_ENABLED, $storeScope);
        if ($active==0 ){
            return $result;
        }

            $includedTax = $quote->getQuiversIncludedTax();
            if($includedTax > 0) {

                $totals  =  [
                    'code' => 'included_tax',
                    'title' => 'Estimated Included Tax',
                    'value' => $includedTax
                ];
               
                return $totals;
            }
            
          return $result;
    }
    public function getSiteKeys(){
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $curlClient = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\HTTP\Client\Curl::class);
        $scopconfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $sandbox= $scopconfig->getValue('quivers/parameters/quivers_sanndbox', $storeScope);
        $ref_id= $scopconfig->getValue("quivers/parameters/merchant_ref_id", $storeScope);
        if($sandbox == 0) {
            $middlewareurl ='https://middleware.quivers.com/profile/';
        }
        else {
            $middlewareurl ='https://middleware.quiversdemo.com/profile/';
        }
        $curlClient->setOption(CURLOPT_CONNECTTIMEOUT, 9);
        $curlClient->setOption(CURLOPT_TIMEOUT, 10);
   
        $sitedata = [
            "client_type"=> "Magento2",
            "business_refid"=> $ref_id
        ];
        try {
            $curlClient->post($middlewareurl.'retrieve/sitekeys/',json_encode($sitedata));
            $response = $curlClient->getBody();
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $customer_session=$objectManager->get('\Magento\Customer\Model\Session');
            $customer_session->setSiteKey(json_decode($response,true));
        } catch (\Exception $e) {
            throw new \Magento\Framework\Validator\Exception(__($e->getMessage().'Site key APIs not working please check again.'));
        }
    }

    public function getMarketPlaceForCurrency($orderCurrency) {
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        if($orderCurrency) {
            $marketPlaceId = null;
            switch (strtolower($orderCurrency)) {
                case 'usd':
                    $marketPlaceId = $scopeConfig->getValue(self::XML_USD_MARKETPLACE_ID, $storeScope);
                    break;
                case 'eur':
                    $marketPlaceId = $scopeConfig->getValue(self::XML_EUR_MARKETPLACE_ID, $storeScope);
                    break;
                case 'cad':
                    $marketPlaceId = $scopeConfig->getValue(self::XML_CAD_MARKETPLACE_ID, $storeScope);
                    break;
                case 'aud':
                    $marketPlaceId = $scopeConfig->getValue(self::XML_AUD_MARKETPLACE_ID, $storeScope);
                    break;
                case 'jpy':
                    $marketPlaceId = $scopeConfig->getValue(self::XML_JPY_MARKETPLACE_ID, $storeScope);
                    break;
                case 'gbp':
                    $marketPlaceId = $scopeConfig->getValue(self::XML_GBP_MARKETPLACE_ID, $storeScope);
                    break;
            }
            return $marketPlaceId;
        }
    }

}
